
from cocos.euclid import Vector2

# http://www.pygame.org/project/752/
# Clinton (clintonselke)
# modified for pyweek

class QuadTree:
    class Node:
        def __init__(self, objects, limit, max_depth):
            if max_depth > 1 and len(objects) > limit:
                self = QuadTree.BranchNode(objects, limit, max_depth)
            else:
                self = QuadTree.LeafNode(objects)
    
    class BranchNode(Node):
        def __init__(self, objects, limit, max_depth):
            self.pivot = Vector2(0.0, 0.0)
            for obj in objects:
                self.pivot += obj.pos
            self.pivot *= 1.0 / len(objects)
            quad = [[],[],[],[]]
            for obj in objects:
                if obj.pos.x < self.pivot.x + obj.size:
                    if obj.pos.y < self.pivot.y + obj.size:
                        quad[0].append(obj)
                    if obj.pos.y > self.pivot.y - obj.size:
                        quad[1].append(obj)
                if obj.pos.x > self.pivot.x - obj.size:
                    if obj.pos.y < self.pivot.y + obj.size:
                        quad[2].append(obj)
                    if obj.pos.y > self.pivot.y - obj.size:
                        quad[3].append(obj)
            self.quad = [QuadTree.Node(quad[0], limit, max_depth-1),
                         QuadTree.Node(quad[1], limit, max_depth-1),
                         QuadTree.Node(quad[2], limit, max_depth-1),
                         QuadTree.Node(quad[3], limit, max_depth-1)]
    
    class LeafNode(Node):
        def __init__(self, objects):
            self.objects = objects
            for i in range(len(objects)-1):
                obj1 = objects[i]
                for j in range(i+1, len(objects)):
                    obj2 = objects[j]
                    if obj1 != obj2:
                        s2 = obj1.size + obj2.size
                        s2 *= s2
                        if (obj1.pos - obj2.pos).magnitude_squared() < s2:
                            obj1.collide(obj2)
    
    def __init__(self, objects, limit, max_depth):
        self.root = QuadTree.Node(objects, limit, max_depth)


# Performs QuadTree collisions as a function rather than class.
def detect_collisions_quadtree(objects, limit=5, max_depth=5):
    if len(objects) <= limit or max_depth == 0:
        for i in range(len(objects)-1):
            obj1 = objects[i]
            for j in range(i+1, len(objects)):
                obj2 = objects[j]
                if obj1 != obj2:
                    s2 = obj1.size + obj2.size
                    s2 *= s2
                    if (obj2.pos - obj1.pos).magnitude_squared() < s2:
                        obj1.collide(obj2)
    else:
        quad = [[],[],[],[]]
        pivot = Vector2(0.0,0.0)
        for obj in objects:
            pivot += obj.pos
        pivot *= 1.0 / len(objects)
        for obj in objects:
            if obj.pos.x - obj.size < pivot.x:
                if obj.pos.y - obj.size < pivot.y:
                    quad[0].append(obj)
                if obj.pos.y + obj.size > pivot.y:
                    quad[1].append(obj)
            if obj.pos.x + obj.size > pivot.x:
                if obj.pos.y - obj.size < pivot.y:
                    quad[2].append(obj)
                if obj.pos.y + obj.size > pivot.y:
                    quad[3].append(obj)
        detect_collisions_quadtree(quad[0], limit, max_depth-1)
        detect_collisions_quadtree(quad[1], limit, max_depth-1)
        detect_collisions_quadtree(quad[2], limit, max_depth-1)
        detect_collisions_quadtree(quad[3], limit, max_depth-1)

def detect_collisions_regular(objects):
    for i in range(len(objects)-1):
        obj1 = objects[i]
        for j in range(i+1, len(objects)):
            obj2 = objects[j]
            if obj1 != obj2:
                s2 = obj1.size + obj2.size
                s2 *= s2
                if (obj2.pos - obj1.pos).magnitude_squared() < s2:
                    obj1.collide(obj2)
