from cocos.sprite import Sprite
from cocos.euclid import Vector2

# http://www.red3d.com/cwr/boids/
# http://www.vergenet.net/~conrad/boids/pseudocode.html

# http://www.red3d.com/cwr/steer/gdc99/
# http://www.red3d.com/cwr/steer/LeaderFollow.html

from random import random

class Boid(Sprite):
    

    def __init__(self, flock, name, cfg):
        Sprite.__init__(self, name)
        
        # Which flock to I belong to?
        self.flock = flock
        
        rulemodifyoffset = {
                                "goto":(random()-0.5)*cfg.rules_modify["goto"],
                                "velocity":(random()-0.5)*cfg.rules_modify["velocity"],
                                "target":(random()-0.5)*cfg.rules_modify["target"],
                                "away":(random()-0.5)*cfg.rules_modify["away"],
                                "speed":(random()-0.5)*cfg.rules_modify["speed"],
                                "group":(random()-0.5)*cfg.rules_modify["group"],
                                "seperate":(random()-0.5)*cfg.rules_modify["seperate"]
                                }
            
  
        # rule defaults
        self.GROUP_DISTANCE =           cfg.rules["group"]#*()*cfg.rules_modify["group"]#100#*100        
        self.GOTO_CENTER_FACTOR =       cfg.rules["goto"]#*cfg.rules_modify["goto"]# 0.3#0.01
        self.VELOCITY_MATCH_FACTOR =    cfg.rules["velocity"]#*cfg.rules_modify["velocity"]#1.0/8.0#0.01
        self.TARGET_MATCH_FACTOR =      cfg.rules["target"]#*cfg.rules_modify["target"]#0.3
        self.KEEP_AWAY_FACTOR =         cfg.rules["away"]#*cfg.rules_modify["away"]#0.3#0.01#0.5       
        self.MAX_SPEED =                cfg.rules["speed"]#*cfg.rules_modify["speed"]#100.0#*100.0 # Factor squared
        self.MIN_SEPERATE =             cfg.rules["seperate"]#*cfg.rules_modify["seperate"]#10.0#*20.0#25*25 # Factor squared in pixels
        
       
        self.GOTO_CENTER_FACTOR     += rulemodifyoffset["goto"]
        self.VELOCITY_MATCH_FACTOR  += rulemodifyoffset["velocity"]
        self.TARGET_MATCH_FACTOR    += rulemodifyoffset["target"]
        self.KEEP_AWAY_FACTOR       += rulemodifyoffset["away"]
        self.GROUP_DISTANCE         += rulemodifyoffset["group"]           
        self.MAX_SPEED              += rulemodifyoffset["speed"]
        self.MIN_SEPERATE           += rulemodifyoffset["seperate"]
        
        self.velocity = Vector2(0, 0) # current
        self.velocity_dash = Vector2(0, 0) # next
        self.pos = Vector2(0, 0)
        
        #print self.GOTO_CENTER_FACTOR,self.VELOCITY_MATCH_FACTOR

    def rule1_center_of_mass(self, other_boids):
        center = Vector2(0, 0)
        
        counter = 0
        for boid in other_boids:
            if boid is self:
                continue
            #if (self.pos - boid.pos).magnitude_squared() < self.GROUP_DISTANCE:
            if (self.pos - boid.pos).magnitude() < self.GROUP_DISTANCE:
                center += boid.position
                counter += 1
            
        if counter>0:
            center /= counter
            return (center - self.position)
        
        return center

    def rule2_keep_away(self, other_boids):
        velocity_offset = Vector2(0, 0)
        
        #closest = 0
        for boid in other_boids:
            if boid is self:
                continue
            #if (self.pos - boid.pos).magnitude_squared() < self.MIN_SEPERATE:
            if (self.pos - boid.pos).magnitude() < self.MIN_SEPERATE:
                #if closest <8:
                velocity_offset -= (boid.pos - self.pos)
                    #closest += 1
                
        return velocity_offset

    def rule3_match_velocity(self, other_boids):
        velocity_offset = Vector2(0, 0)
        if len(other_boids) <= 1:
            return velocity_offset
        
        for boid in other_boids:
            if boid is self:
                continue
            velocity_offset += boid.velocity
            
        velocity_offset /= len(other_boids) - 1
        
        return (velocity_offset - self.velocity)
    
    def rule4_tend_to_place(self, place): #kinda like seek?
        return (place - self.pos) 
    
    def VelocityUpdate(self, other_boids, target, rulemodify = [1,1,1,1]): 
        """Called for all boids BEFORE position update on ANY boid"""
        
        v1 = Vector2(0,0)
        v2 = Vector2(0,0)
        v3 = Vector2(0,0)
        v4 = Vector2(0,0)
        
        if self.GOTO_CENTER_FACTOR != 0 and rulemodify[0] !=0 :
            v1 = rulemodify[0] * self.rule1_center_of_mass(other_boids) * self.GOTO_CENTER_FACTOR
        
        if self.KEEP_AWAY_FACTOR != 0 and rulemodify[1] !=0 :
            v2 = rulemodify[1] * self.rule2_keep_away(other_boids) * self.KEEP_AWAY_FACTOR
        
        if self.VELOCITY_MATCH_FACTOR != 0 and rulemodify[2] !=0 :
            v3 = rulemodify[2] * self.rule3_match_velocity(other_boids) * self.VELOCITY_MATCH_FACTOR
        
        if self.TARGET_MATCH_FACTOR != 0 and rulemodify[3] !=0 :
            v4 = rulemodify[3] * self.rule4_tend_to_place(target)* self.TARGET_MATCH_FACTOR
        
        self.velocity_dash = v1 + v2 + v3 + v4

    def PositionUpdate(self, dt):
        self.velocity += self.velocity_dash
        #if self.velocity.magnitude_squared() >  self.MAX_SPEED:
        if self.velocity.magnitude() >  self.MAX_SPEED:
            #print "too fast:",self.velocity
            #self.velocity /= self.velocity.magnitude_squared() /  self.MAX_SPEED
            self.velocity /= self.velocity.magnitude() /  self.MAX_SPEED
            #print "new:",self.velocity
        #if self.velocity.magnitude() > 100:
        #    self.velocity = (self.velocity / self.velocity.magnitude())*100.0
        self.pos += self.velocity*dt
        
        # update the sprite
        self.position = self.pos.x, self.pos.y

