

from cocos.layer import Layer
from cocos.sprite import Sprite
import cocos
from cocos.director import director
from pyglet import clock
from cocos.euclid import Vector2
from fsm import FSM
from humans import Human
from health import Health
import pyglet
from anims import GetAnimSeqFromFile, GetFramesFromFile

import logging
log = logging.getLogger('city')
 
from constants import *
from splode import Splode, Smoke
class City(Sprite, Health, FSM):

    is_event_handler = True # for actions to work
    
    city_states = [
              "idle",
              "annoyed",
              "angry",
              "furious",
              ]
    city_inputs = [
              "crash", # Plane crash
              "death", # Human dead
              ]
    
    def __init__(self, world, name, cfg, human_cfg):
        self.house_image =          GetFramesFromFile(cfg.filename,cfg.frames) 
        

        Sprite.__init__(self,self.house_image[0])
        ax,ay = self.house_image[0].frames[0].image.anchor_x, self.house_image[0].frames[0].image.anchor_y
        print ax,ay
        for img in self.house_image:
            img.frames[0].image.anchor_x = ax
            img.frames[0].image.anchor_y = ay
        self.house_image_idx = 0
        self.position =             cfg.position
        self.human_cfg =            human_cfg
        self.attack_distance =      cfg.attack_distance
        self.attack_frequency =     cfg.attack_frequency
        self.last_attack_time =     0
        self.original_position = self.position
        self.DEATHS_PER_ANNOYANCE = cfg.death_annoy
        self.CRASH_PER_ANNOYANCE =  cfg.crash_annoy
        
        FSM.__init__(self)
        Health.__init__(self, world.quadtree, type = "city", max_health = cfg.default_health)
        
        self.world = world
        self.name = name

        
        self.humans = []

        self.deaths = 0
        self.crashes = 0
        
        self.nest = None
        
        self.last_spawn_time = 0
        
        self.city_spawn = {
                  "idle": 1,
                  "annoyed": 3,
                  "angry":6,
                  "furious":10,
                  }
        
        self.add_state("idle",      "annoy",        "annoyed",  self.do_spawn)
        self.add_state("annoyed",   "annoy",        "angry",    self.do_spawn)
        self.add_state("angry",     "annoy",        "furious",  self.do_spawn)
        self.add_state("furious",   "annoy",        "furious",  self.do_spawn)
        
        self.add_state("idle",      "destroy_city", "annoyed",  self.do_spawn)
        self.add_state("annoyed",   "destroy_city", "angry",    self.do_spawn)
        self.add_state("angry",     "destroy_city", "furious",  self.do_spawn)
        self.add_state("furious",   "destroy_city", "furious",  self.do_spawn)
        
        self.add_state("annoyed",   "death",        "annoyed",  self.do_count_death)
        self.add_state("angry",     "death",        "angry",    self.do_count_death)
        self.add_state("furious",   "death",        "furious",  self.do_count_death)
        
        self.add_state("annoyed",   "kill_bird",    "annoyed",  self.do_count_kill)
        self.add_state("angry",     "kill_bird",    "angry",    self.do_count_kill)
        self.add_state("furious",   "kill_bird",    "furious",  self.do_count_kill)
        
        self.add_state("idle",      "crash",        "idle",     self.do_count_crash)
        self.add_state("annoyed",   "crash",        "annoyed",  self.do_count_crash)
        self.add_state("angry",     "crash",        "angry",    self.do_count_crash)
        self.add_state("furious",   "crash",        "furious",  self.do_count_crash)
        
        # send some people home?
        self.add_state("annoyed",   "kill_nest",    "idle",     self.do_killed_nest)
        self.add_state("angry",     "kill_nest",    "annoyed",  self.do_killed_nest)
        self.add_state("furious",   "kill_nest",    "annoyed",  self.do_killed_nest)
        
        if DEBUG:
            self.state_label = cocos.text.Label('City!')
            self.add(self.state_label)
            self.schedule(self.update_state_label)
                    
    def update_state_label(self,dt):
        self.state_label.element.text = str(self.state)

    def update(self,dt):
        
            
        FSM.update(self, dt)
        for human in self.humans:
            human.update(dt)
    
    def add_human(self, human):
        log.debug(self.state+": Adding human")
        self.humans.append(human)
        self.world.level.maplayer.add(human, z=10)
        
    def do_count_death(self):
        self.do_spawn()
        self.deaths += 1
        if self.deaths > self.DEATHS_PER_ANNOYANCE:
            self.deaths = 0
            self.execute_state("crash")
        else:
            self.activity = self.do_idle
        
    def do_count_crash(self):
        self.crashes += 1
        log.info(self.name+" ["+self.state+"]: do_count_crash %r"%self.crashes)
        if self.crashes > self.CRASH_PER_ANNOYANCE:
            log.info(self.name+" ["+self.state+"]: CRASH_PER_ANNOYANCE reached %r"%self.crashes)
            self.crashes = 0
            self.execute_state("annoy")
        else:
            self.activity = self.do_idle
            
    def do_count_kill(self):
        if self.deaths > 0:
            self.deaths -= 1
            
        self.activity = self.do_idle
            
    def do_killed_nest(self):
        self.nest = None
        for human in self.humans:
             human.nest = None
           
    def do_idle(self):
        pass
            
    def do_spawn(self):
        time = clock.time.time()
        
        spawn_new = self.city_spawn[self.state] - len(self.humans)
        if spawn_new >0 and time-self.last_spawn_time>1:
            #for new in range(spawn_new):
            self.last_spawn_time = time
            human = Human(self.world.quadtree, self.human_cfg, city = self)

            human.pos = Vector2(int(self.position[0]), int(self.position[1]))#self.city_image.position
            human.set_state("idle")
            if self.state == "furious":
                log.debug( self.state+": setting nest %s"%self.nest)
                human.nest = self.nest

                
            self.add_human(human)
            log.info( self.name+" - "+ self.state+": spawning %s"%spawn_new)
            
        for human in self.humans:
            human.execute_state("city_"+self.state)
        
    def hurt(self):
        Health.hurt(self)
        max_frames = len(self.house_image)
        
        if int(self.health) <max_frames and int(self.health) >=0:
            frame = (max_frames-1)-int(self.health)
            if frame != 0:
                smoke = Smoke(dx=0, dy=0.1, gravity =  (0, 0.1, 0), myrate=10, spread =  (0, 1, 0))

                lookup = [0, self.width*0.25, 0, -self.width*0.25, -self.width*0.55]
                smoke.position = lookup[frame], -10
                self.add(smoke, z = -15)

            #log.info("frame:"+str(frame))
            self.image = self.house_image[frame]
        
    def die(self):
        self.world.cities.remove(self)     
        self.world.TellCities("destroy_city")
        
        for human in self.humans:
            human.city = None
            self.world.humans.append(human)
        
        self.humans = []
            
        Health.die(self, True)  
        
        