from boid import Boid
from cocos.euclid import Vector2
import logging
log = logging.getLogger('cmplxboid')

class ComplexBoid(Boid):
    def __init__(self,flock , name, cfg):
        
        # set max_speed
        Boid.__init__(self, flock, name, cfg)
        
        self.acc = Vector2(0, 0) # current
        
        self.MAX_STEER_FORCE =          cfg.max_steer_force #100.0       
        self.SLOW_DISTANCE =            cfg.slow_distance #5.0
        self.AVOID_INFLUENCE =          cfg.target_influence
        self.TARGET_INFLUENCE =         cfg.avoid_influence
        self.MIN_INFLUENCE_DISTANCE =   cfg.avoid_influence_dist
        
        self.path_radius  = 100.0
        
        self.path = None
        self.target =  None
        self.arrive = False
        self.velocity_dash = Vector2(0, 0)
        
        #print "CPX", name


    def steer(self, target, slowdown):
        #print "steer", target, slowdown
        steer = Vector2(0, 0)
        desired = target-self.pos 
        dist = desired.magnitude()

        if dist > 0:
            #print "steer",dist
            desired.normalize()

            if slowdown and (dist < self.SLOW_DISTANCE):
                desired *= self.MAX_STEER_FORCE*(dist/self.SLOW_DISTANCE)
            else:
                desired *= self.MAX_STEER_FORCE
                
            steer = desired - self.velocity

            if steer.magnitude() >  self.MAX_STEER_FORCE:
                steer /= steer.magnitude() /   self.MAX_STEER_FORCE

        return steer
    
    # Adapted from Daniel Shiffman <http://www.shiffman.net>
    
    def GetNormalPoint(self, point, a, b):
        ap = point - a
        ab = b - a
        ab.normalize()
        ab *= ap.dot(ab)

        return a+ab;
        
    def follow_path(self, path, path_radius, loop = False):
        predict = self.pos + self.velocity.normalized()*20
        target = None
        dir = None
        best = 1000000
        
        last = path[0]
         
        length = len(path) 
        if loop is False:
            length -= 2
            
        for i in range(length):
            a = path[i]
            b = path[(i+1)% length]
    
            normal = self.GetNormalPoint(predict,a,b)
            da = (normal - a).magnitude()
            db = (normal - b).magnitude()
            
            line = b-a

            if da + db > line.magnitude()+1:
                normal = b

                a = path[(i+1) % length]
                b = path[(i+2) % length]
                line = b - a
          
            d = (predict-normal).magnitude()
            if (d < best):
                best = d
                target = normal
                dir = line
                dir.normalize()
                dir *= 20
                
        #if self.type == "human":
        #    log.info("best path:"+str((best, path_radius, dir)))
        if best > path_radius or self.velocity.magnitude() < 0.1:
            target += dir
            return self.steer(target, False)
        else:
            return Vector2(0,0)
    
 
    def AvoidUpdate(self, avoid):
        if avoid is not None and  (avoid - self.pos).magnitude() < self.MIN_INFLUENCE_DISTANCE:        
            #log.info("Avoid :"+str(avoid))
            steer = self.steer(avoid, False)#*30000
            self.acc -= steer*self.AVOID_INFLUENCE #0.01
                
                
    def VelocityUpdate(self, loopPath = False): 
        if self.path is not None:
            steer = self.follow_path(self.path, self.path_radius, loopPath)
            self.acc += steer*self.TARGET_INFLUENCE
            #if self.type == "human":
            #    log.info("Following path:"+str((self.TARGET_INFLUENCE, steer, self.acc)))
                    
        if self.target is not None:
            steer = self.steer(self.target, self.arrive)
            self.acc += steer*self.TARGET_INFLUENCE#0.015
            #print "VelocityUpdate", self.target, self.acc,self.velocity
        
    def PositionUpdate(self, dt):
        #print "PositionUpdate", dt, self.acc
        #self.velocity += self.acc*dt*100
        self.velocity += self.acc
        #print "PositionUpdate", self.target, self.acc,self.velocity
        Boid.PositionUpdate(self, dt)
        #if self.type == "human":
        #    log.info("PositionUpdate:"+str((self.velocity, self.acc, self.pos)))
        
        self.acc = Vector2(0, 0)
