
from bird import Bird
from cocos.euclid import Vector2
from cocos.layer import Layer
from cocos.batch import BatchNode
from random import randrange, random
from pixel_collide import collide
from pyglet import clock

import logging
log = logging.getLogger('flock')

from anims import GetAnimSeqFromFile
import weakref
from cocos.cocosnode import CocosNode

class Flock(CocosNode):
    def __init__(self, world, cfg, numboids):
        CocosNode.__init__(self)
        self.boidimage = GetAnimSeqFromFile(cfg.filename,(9,6))
        self.boid_splat = GetAnimSeqFromFile("data/mini_images/birdsplat.png",(9,9))
        
        self.cfg = cfg
        self.target = Vector2(150,150)
        self.boids = []
        self.world = world
        
        for b in range(numboids):
            self.AddToFlock()

         
        self.center = Vector2(150,150)
        self.attached = True
        self.wander = False

    def UpdateFlockCenter(self):
        self.center = Vector2(0,0)

        if len(self.boids) <= 0:
            return
        
        
        for boid in self.boids:
            self.center += boid.position
                
        self.center /= len(self.boids)
       
    def AddToFlock(self, pos = None):
        if pos is None:
            pos = self.target.x,self.target.y#self.position
            
        #print "making boid",rulemodifyoffset
        bird = Bird(self.world.quadtree, self,  self.boidimage, self.boid_splat, self.cfg)
        bird.pos = Vector2(pos[0]+randrange(100,200), pos[0]+randrange(100,200))
        self.boids.append(bird)
        
        # add to level
        self.world.level.maplayer.add(bird,z=10)
        self.world.birds.append(weakref.ref(bird))

    def Merge(self, flock):
        log.info("Merge!")
        for boid in flock.boids:
            boid.flock = self
            #boid.parent = self
            self.boids.append(boid)
            #self.add(boid)

    def PickRandomTarget(self, dt):
        self.target = Vector2(randrange(100,2000),randrange(100,1000))
            
    def UpdateFlock(self, dt):
        if self.attached is False and self.wander is False:
            self.schedule_interval(self.PickRandomTarget , 10)
            self.wander = True
            
        for boid in self.boids:
            #print "VelocityUpdate boid", boid
            boid.VelocityUpdate(self.boids, self.target)
            
        for boid in self.boids:
            boid.PositionUpdate(dt)
            #print "PositionUpdate boid", boid.position
     
    