
from cocos.layer import Layer,ColorLayer
from cocos.text import Label
from cocos.director import director
from cocos.actions import *
from cocos.sprite import Sprite

import config
from common import *
from constants import *
import data
import logging
log = logging.getLogger('hud')
import gethigh

class GameOverBG( ColorLayer ):
    def __init__(self, name):
        ColorLayer.__init__(self, 127,127,127,127)
        w, h = director.get_window_size()
        bg= Sprite("data/backgrounds/"+name+"_bg.png")
        bg.position = w/2,h/2
        self.add(bg)

        
class Hud(Layer):
    def __init__(self, world):
        Layer.__init__(self)
        self.world = world
        self.world.push_handlers( self.on_kill_human,  self.on_kill_aircraft, self.on_game_over,\
                                  self.on_flock_size_update, self.on_nest_size_update, self.on_game_win )
        
        width, height = director.get_window_size()
        
        self.score = 0
        self.score_text = Label('Score: 0')       
        self.score_text.position = width*0.1,height-20
        self.add(self.score_text)
                    
        self.birds_in_flock = 0
        self.birds_in_flock_text = Label('Flock: 0')       
        self.birds_in_flock_text.position = width*0.3,height-20
        self.add(self.birds_in_flock_text)
                    
        self.birds_in_nest = 0
        self.birds_in_nest_text = Label('Rookery: 0')       
        self.birds_in_nest_text.position = width*0.5,height-20
        self.add(self.birds_in_nest_text)
        
        self.on_flock_size_update()
        self.on_nest_size_update()

    def update_birds_in_nest_label(self):
        self.birds_in_nest_text.element.text = "Rookery: "+str(self.birds_in_nest)
        log.debug("update_birds_in_nest_label: "+self.birds_in_nest_text.element.text)

    def update_birds_in_flock_label(self):
        self.birds_in_flock_text.element.text = "Flock: "+str(self.birds_in_flock)
        log.info("update_birds_in_flock_label: "+self.birds_in_flock_text.element.text)
                    
    def update_score_label(self):
        self.score_text.element.text = "Score: "+str(self.score)


    def on_kill_human(self):
        self.score += 10
        self.update_score_label()
    def on_kill_aircraft(self):
        self.score += 1000
        self.update_score_label()
    def on_kill_city_part(self):
        self.score += 1000
        self.update_score_label()
    def on_kill_city_complete(self):
        self.score += 5000
        self.update_score_label()
        
    def on_game_over(self):
        w, h = director.get_window_size()
        self.add(GameOverBG("gameover"))
        director.scene.do(Delay(2)+FadeOut(1))
        
        self.do(Delay(3)+CallFunc(director.scene.end))
        gethigh.SubmitScore(config.playername, self.score)
        
    def on_game_win(self):
        w, h = director.get_window_size()
        self.add(GameOverBG("gamewin"))
        director.scene.do(Delay(2)+FadeOut(1))
        
        self.do(Delay(3)+CallFunc(director.scene.end))
        gethigh.SubmitScore(config.playername, self.score)
        

    def on_kill_bird(self):
        self.on_flock_size_update()
        
    def on_kill_nest(self):
        self.birds_in_nest = 0
        self.update_birds_in_flock_label()
        ##self.birds_in_nest_text.
    
    def on_flock_size_update(self):
        self.birds_in_flock = len(self.world.flock.boids)
        log.info("on_flock_size_update: "+str( self.birds_in_flock))
        self.update_birds_in_flock_label()
    
    def on_nest_size_update(self):
        #log.info("on_nest_size_update: "+str( (self.birds_in_nest,self.world.nest.health)))
        self.birds_in_nest = int(self.world.nest.health)
        self.update_birds_in_nest_label()

