
import cocos

import config
from common import *
from constants import *
import data

__all__ = ['Human']

from fsm import FSM

import random
from cocos.sprite import  Sprite
from cocos.actions import *
from cocos.euclid import Vector2
from pyglet import clock

import logging
log = logging.getLogger('human')
 
import math
from extern.primitives import Line
from health import Health

from anims import GetAnimSeqFromFile, GetFramesFromFile
from complex_boid import ComplexBoid

class Human(FSM, Health, ComplexBoid):
    """
       Humans walk from one side of the map to the other
       shooting birds
    """
    def __init__(self, sg, cfg, city = None, name = "data/images/person1.png"):
        
        self.last_fire_time = 0
        self.last_check_shoot_time = 0
        self.attack_distance = cfg.attack_distance
        self.attack_frequency = cfg.attack_frequency
        
        self.walk_left_seq = GetAnimSeqFromFile(cfg.filename,(25,25))
        self.walk_right_seq = GetAnimSeqFromFile("data/mini_images/hunter_walk_right.png",(25,25))#self.walk_left_seq.get_transform(flip_x = True)
        self.face_left = True
        self.shoot_left_seq = GetFramesFromFile(cfg.shoot_file,(25,25))

        ComplexBoid.__init__(self, None, self.walk_left_seq, cfg)
        FSM.__init__(self)
        Health.__init__(self, sg, type = "human", max_health = cfg.default_health)
        
        self.city = city
        self.world = city.world
        #self.mental_states = ["idle", "active", "attack"]
        self.mental_state = "idle"


        self.add_state("idle",              None,               "idle",                 self.do_idle)
        self.add_state("idle",              "idle",             "idle",                 self.do_idle)
       
        self.add_state("idle",              "city_annoyed",     "active",               self.do_find_birds)
        self.add_state("idle",              "city_angry",       "attack",               self.do_find_birds)
        self.add_state("idle",              "city_furious",     "attack",               self.do_find_rookery)       
  
        self.add_state("idle",              "found_flying",     "idle_check_shoot",     self.do_check_shoot)
        self.add_state("idle",              "found_rookery",    "idle",                 self.do_idle)
        
        self.add_state("idle_check_shoot",  "shouldnt_shoot",   "idle",                 self.do_cancel_shot)
        self.add_state("idle_check_shoot",  "should_shoot",     "idle_shoot",           self.do_shoot)
        
        self.add_state("idle_shoot",        "cant_shoot",       "idle",                 self.do_cancel_shot)
        self.add_state("idle_shoot",        "kill",             "idle",                 self.do_idle)
        
        
        self.add_state("active",            "city_angry",       "attack",               self.do_find_rookery)
        self.add_state("active",            "city_furious",     "attack",               self.do_find_rookery)
        
        self.add_state("active",            "found_flying",     "active_aim",           self.do_aim_at_flying)
        self.add_state("active_aim",        "should_shoot",     "active_shoot",         self.do_shoot)
        self.add_state("active_shoot",      "cant_shoot",       "active",               self.do_cancel_shot)
        self.add_state("active_shoot",      "kill",             "active",               self.do_idle)
        
        self.add_state("active",            "found_rookery",    "active",               self.do_find_birds)
 

        self.add_state("attack",            "found_flying",     "attack_aim",           self.do_aim_at_flying)        
        self.add_state("attack_aim",        "should_shoot",     "attack_shoot",         self.do_shoot)
        self.add_state("attack_shoot",      "cant_shoot",       "attack",               self.do_cancel_shot)       
        self.add_state("attack_shoot",      "kill",             "attack",               self.do_idle)
        
        self.add_state("attack",            "found_rookery",    "attack_shoot_rookery", self.do_shoot_rookery)
        
        self.add_state("attack_shoot_rookery", "cant_shoot",    "attack",               self.do_cancel_shot)
        self.add_state("attack_shoot_rookery", "kill",          "attack",               self.do_idle)
 
        
        self.bird = None
        self.nest = None
        
        self.path =  []
        for point in cfg.spawn["path"]:
            self.path.append(Vector2(point[0], point[1]))
        self.path_radius  = cfg.spawn["path_radius"]
        
        log.info("path:"+str(self.path))
                
        self.shoot_line = None
        self.pos = Vector2(0,0)
        if DEBUG:
            self.state_label = cocos.text.Label('')
            self.state_label.position = 0, -10
            self.add(self.state_label)        
            self.schedule(self.update_state_label)
           
    def update_state_label(self,dt):
        self.state_label.element.text = str((self.state, self.pos))
        
    def die(self):
        if self.city is not None:
            self.city.humans.remove(self)     
        else:
            self.world.humans.remove(self) 
            
        Health.die(self)  
        
    def set_closest_bird(self, bird):
        if bird is None:
            self.bird = None
        elif self.bird_in_range(bird, self.attack_distance) and (self.bird is None or ( bird != self.bird and  self.dist_to_bird(bird) < self.dist_to_bird(self.bird))):
            log.debug(self.state+  " new target bird %s"%bird.pos)
            self.bird = bird
            
    def dist_to_bird(self, bird):
        pos = Vector2(self.x,self.y)
        
        return (pos - bird.pos).magnitude()
    
    def bird_in_range(self, bird, range):
        if bird is None:
            return False
        return self.dist_to_bird(bird) < range
    
    def fire_gun(self, target):  
        time = clock.time.time()
        if time - self.last_fire_time > self.attack_frequency:
            log.debug(self.state+  " BANG %s %s %s"%( time, target.type, target.health)     )
            
            if target.type == "nest" and target.health == 0:
                self.nest = None
                self.execute_state("kill")
                return
            
            self.last_fire_time = time
            self.shoot_line = Line( a=self.position, b=target.position, z=20, color=(0.2,0,0,1))    
           
            # 100% accuracy atm
            if target.hurt():
                if target.type == "nest":
                    self.nest = None
                    
                log.info(self.state+  " KILL %s "%(target.type)     )
                self.kills += 1
                # tell city
                self.execute_state("kill")
                self.world.dispatch_event("on_kill_"+target.type)

                if self.city is not None:                    
                    self.city.execute_state("kill_"+target.type)
                
                return
            
            self.execute_state("cant_shoot")
            
    
    def look_for_birds(self):
        time = clock.time.time()
        log.debug("look_for_birds "+self.state)
        # Wander around and take pot shots
        if self.bird is not None and time - self.last_check_shoot_time>self.attack_frequency:
            self.last_check_shoot_time = time
            
            self.execute_state("found_flying")
            return True
        return False
 

    def do_cancel_shot(self):
        self.bird = None
        self.activity = self.do_idle

    def do_idle(self):
        log.debug("do_idle "+self.state)
        ComplexBoid.VelocityUpdate(self, True)
        
        if not self.look_for_birds():
            if self.state == "attack" and self.nest is not None:
                self.do_find_rookery()

    def do_find_birds(self):
        log.debug("do_find_birds "+self.state)
        ComplexBoid.VelocityUpdate(self, True)
        self.look_for_birds()
    
    def do_find_rookery(self):
        log.debug("do_find_rookery "+self.state+" "+str(self.nest))
        ComplexBoid.VelocityUpdate(self, True)
        if not self.look_for_rookery():
            self.look_for_birds()
        
    def look_for_rookery(self):
        log.debug("look_for_rookery "+self.state+" "+str(self.nest))
        # Look for rookery and take pot shots
        if self.nest is not None and self.nest.health == 0:
            self.nest = None
            self.execute_state("cant_shoot")
            
        if self.nest is not None:
            time = clock.time.time()
            if time - self.last_check_shoot_time>1:
                
                dist =  Vector2(self.nest.x - self.x, self.nest.y - self.y).magnitude()
                if dist < self.attack_distance:
                    self.last_check_shoot_time = time
                    log.debug("Nest in sight! "+str(dist))
                    
                    self.execute_state("found_rookery")
                    return True
                else:
                    log.debug("Too far:"+str(dist))
        else:
            if self.state == "attack_shoot_rookery":
                self.execute_state("cant_shoot")
                
            log.debug("do_find_rookery "+self.state+" "+"no rookery")
        return False
    
    def do_goto(self):
        log.info("do_goto "+self.state)
        
    def do_check_shoot(self):
        log.info("do_check_shoot "+self.state)
        
        self.activity = None
        if random.randint(0,10) > 8:
            self.do(Delay(1.5)+CallFunc(self.execute_state,"should_shoot"))
            #self.execute_state("should_shoot")
        else:
            self.do(Delay(1.5)+CallFunc(self.execute_state,"shouldnt_shoot"))
            #self.execute_state("shouldnt_shoot")
            

    def do_shoot(self):
        
        if self.bird_in_range(self.bird, self.attack_distance):
            # make sound
            log.info("do_shoot "+self.state+" shooting")
            self.fire_gun(self.bird)
        else:
            log.info("do_shoot "+self.state+" cant shoot")
            self.execute_state("cant_shoot")
    
    def do_aim_at_flying(self):
        log.info("do_aim_at_flying "+self.state)
        self.activity = None
        self.do(Delay(1.5)+CallFunc(self.execute_state,"should_shoot"))
    
    def do_shoot_rookery(self):
        log.debug("do_shoot_rookery "+self.state)
        if self.nest is not None:
            # make sound
            self.fire_gun(self.nest)
        else:
            self.execute_state("cant_shoot")

    
    def update(self, dt):
        self.shoot_line = None
        if self.activity is not None:
            ComplexBoid.PositionUpdate(self, dt)
        Health.update(self, dt)
        FSM.update(self, dt)
        
        self.update_image()
        
    def draw(self):
        
        ComplexBoid.draw(self)
        
        
        if self.shoot_line is not None:
            self.shoot_line.render()
            
    def update_image(self):
        

        target = Vector2(0,0)
        shooting = True
        if self.bird is not None:
            target = self.bird.pos
        elif self.nest is not None:
            target = Vector2(self.nest.x, self.nest.y)
        else:
            shooting = False

        self.position = self.pos
        
        # Walking
        if shooting is False:
            
            #self.position = self.pos.x+self.width/2, self.pos.y-self.height/2
            if self.velocity.x >0:
                if self.face_left is None or self.face_left is True:
                    self.image = self.walk_right_seq
                    self.face_left = False
                self.position = self.pos.x, self.pos.y-self.height/2
                
            if self.velocity.x <0 and (self.face_left is None or self.face_left is False):
                self.image = self.walk_left_seq
                self.face_left = True
        else:# move gun
            self.face_left = None 
            # To the left or right?
            vec = self.pos-target
            deg = (math.degrees(math.atan2(vec.y, vec.x)+math.pi)-90)
            
            frame = 3- abs(int((deg/90)*4))
            if frame >3:
                frame = 3
            if frame <0:
                frame = 0
            if deg <0 :     
                          
                self.image = self.shoot_left_seq[frame].get_transform(flip_x = True)
                self.position = self.pos.x+self.width/2, self.pos.y-self.height/2
                
            if deg > 0:

                self.image = self.shoot_left_seq[frame]
                self.position = self.pos.x-self.width/2, self.pos.y-self.height/2
        
