if __name__ == "__main__":
    # for testing
    import sys,os
    sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

import yaml

import config
from common import *
from constants import *
import data

import logging
log = logging.getLogger('item_cfg')

from random import random

__all__ = ['BuildItemConfigTable', 'SaveItemConfigTable', 'GetItemConfigTable']

_ItemConfigTable = None
def BuildItemConfigTable(filename, f = None):
    if f is None:
        f = data.load_file(filename)
    d = f.read()
    f.close()
    global _ItemConfigTable
    _ItemConfigTable = yaml.load(d)
    return _ItemConfigTable
    
def SaveItemConfigTable(olt, filename):
    d = yaml.dump(olt)
    f = data.load_file(filename, "w")
    f.write(d)
    f.close

def GetItemConfigTable():
    return _ItemConfigTable

##########################################
############# Config Classes #############
##########################################


class ItemConfigTable(yaml.YAMLObject):
    yaml_tag = u'!ItemConfigTable'
    def __init__(self, name, planes, birds, humans, cities, nest, world):
        self.name = name
        self.planes = planes
        self.birds = birds
        self.humans = humans
        self.cities = cities
        self.nest =  nest
        self.world = world


# Birds
class Bird(yaml.YAMLObject):
    yaml_tag = u'!Bird'
    def __init__(self, name, filename, attack_distance, default_health, rules, rules_modify):
        self.name = name
        self.filename = filename
        self.rules = rules
        self.rules_modify = rules_modify
        self.attack_distance = attack_distance
        self.default_health = default_health

# Nest
class Nest(yaml.YAMLObject):
    yaml_tag = u'!Nest'
    def __init__(self, name, filename, default_health, max_in_nest, grow_interval, grow_multiplier, spawn_interval):
        self.name = name
        self.filename = filename
        self.default_health = default_health
        self.max_in_nest =  max_in_nest
        self.grow_interval = grow_interval
        self.grow_multiplier = grow_multiplier
        self.spawn_interval = spawn_interval
        
# Humans
class Human(yaml.YAMLObject):
    yaml_tag = u'!Human'
    def __init__(self, name, filename, attack_distance, default_health):
        self.name = name
        self.filename = filename
        self.attack_distance = attack_distance
        self.default_health = default_health
 
# Plane - specifies types of planes and their stats
class Plane(yaml.YAMLObject):
    yaml_tag = u'!Plane'
    def __init__(self, name, filename, default_health, spawn, avoid_influence, avoid_influence_dist, 
                 stall_speed, fall_accel):
        self.name = name
        self.filename = filename
        self.spawn = spawn      
        self.default_health = default_health
        self.avoid_influence = avoid_influence
        self.avoid_influence_dist = avoid_influence_dist
        self.stall_speed = stall_speed
        self.fall_accel = fall_accel
        
# Cities
class City(yaml.YAMLObject):
    yaml_tag = u'!City'
    def __init__(self, name, filename, attack_distance, default_health, position, death_annoy, crash_annoy):
        self.name = name
        self.filename = filename
        self.attack_distance = attack_distance
        self.default_health = default_health
        self.position = position
        self.death_annoy = death_annoy
        self.crash_annoy = crash_annoy





##########################################
############## Testing ###################
##########################################


DEFAULT_TEST_FILE="levels/test.yaml"
def testSave():
    objs = [
            Plane("SmallPlane", "data/images/plane0.png"),
            Plane("MediumPlane", "data/images/plane1.png"),
            Plane("LargePlane", "data/images/plane2.png"),
            Plane("Baloon", "data/images/baloon.png"),
            Bird("Bird", "data/images/bird.png", 100),
            Human("Human", "data/images/human.png", 100),
            City("CityOne", "data/images/city.png", 50),
            City("CityTwo", "data/images/city.png", 60),
            City("CityThree", "data/images/city.png", 70),
            City("CityFour", "data/images/city.png", 85),
            ]
    
    table = ItemConfigTable("LevelTest", objs)
    SaveItemConfigTable(table, DEFAULT_TEST_FILE)
    
def testLoad():
    olt = BuildItemConfigTable(DEFAULT_TEST_FILE) 
    d = yaml.dump(olt)
    print d
    objs =  yaml.load(d)
    print objs
    
    #pri
    
if __name__ == "__main__":
    #testSave()
    
    testLoad()


