from pyknic.resources.tiledtmxloader import ImageLoaderPyglet
from pixel_collide import collide, SpriteCollision, Rect

from PIL import Image
from PIL import TgaImagePlugin
from PIL import PngImagePlugin

from cocos.tiles import ScrollingManager, RectCell, RectMapLayer#, Tile, TileSet
from cocos.layer import Layer
from cocos.sprite import Sprite
from cocos.layer import ColorLayer

import logging
log = logging.getLogger('level:BG')

import time

class QCollSprite(Sprite):
    def __init__(self, image):
        Sprite.__init__(self, image)

        self.collision = SpriteCollision(self)


class Level(Layer):
    def __init__(self, 
                 tile_size = (32,32),
                 mapName = "Level", 
                 data_file = "data/images/level.tga", 
                 tile_file = "data/images/tiles.png"):
        
        Layer.__init__(self)
        self.notile = None
        
        self.bl_layer = ColorLayer(196,230,255,255)
        self.add(self.bl_layer)
        
        self.tile_width, self.tile_height = tile_size
        
        t = time.time()
        self.level_data = self.LoadLevelDataFromImage(data_file)
        lt = time.time() - t
        log.info( 'LoadLevelDataFromImage took: %f' % lt)
        
        t = time.time()
        self.tile_set = self.GetTileSet(tile_file, self.tile_width, self.tile_height)
        lt = time.time() - t
        log.info( 'GetTileSet took: %f' % lt)
        
        t = time.time()
        self.map = self.BuildMap(self.tile_set, self.level_data, self.tile_width, self.tile_height)
        lt = time.time() - t
        log.info( 'BuildMap took: %f' % lt)
        
        t = time.time()
        self.maplayer = RectMapLayer(mapName, self.tile_width, self.tile_height,  self.map)
        lt = time.time() - t
        log.info( 'RectMapLayer took: %f' % lt)
        #self.maplayer.set_view(0,200,600,200)
        class b():
            width=600
            height=200
        self.scroller = ScrollingManager()
        self.scroller.add(self.maplayer)
        #self.scroller.y += 200
        self.scroller.x += 8
        #self.scroller.height = 300
        self.add(self.scroller)
        self.SetFocus(100,100)
        
    def SetFocus(self, x, y):
        #rgb = self.bl_layer.
        amt = (1.0-float(y)/float(self.maplayer.px_height))
        self.bl_layer.color =  196.0*amt,230.0*amt,255.0
        
        #log.info("bl_layer.opacity:"+str(self.bl_layer.opacity))
        
        self.scroller.set_focus(x, y)
        
    def BuildMap(self, tile_set, packed_data, tile_w, tile_h):
        width , height, level_data = packed_data
        log.info(str(len(tile_set)))
        log.info(str((width , height, level_data) ))
        
        self.notile = RectCell(-100,-100, 0, 0, (-1), tile_set[0])
        map = []
        for x in xrange(width):
            row = []
            for y in xrange(height):  
                data = level_data[x][y]     
                tile =  tile_set[data[0]]   
                if tile == 0:
                    pass # row.append(self.notile)
                else:
                    newtile = RectCell(x,y, tile_w, tile_h, (0), tile)
                    row.append(newtile)
            
            map.append(row)
            
        return map
    
    def GetTileSet(self,path, tile_w, tile_h):
        
        tiles = []
        image_ldr = ImageLoaderPyglet()
    
        t = time.time()
        tilesetimages = image_ldr.load_image_parts(path , 0, 0, tile_w, tile_h)
        lt = time.time() - t
        log.debug( 'load_image_parts for %r images took: %f' % (len(tilesetimages), lt))
        
        t = time.time()
        for img in tilesetimages:
            tt = time.time()
            sp = QCollSprite(img)
            ltt = time.time() - tt
            log.debug( 'QCollSprite 1 images took: %f' % ltt)
            
            tiles.append(sp)
            
        lt = time.time() - t
        log.debug( 'QCollSprite for %r images took: %f' % (len(tilesetimages), lt))
    
        return tiles
    
    def LoadLevelDataFromImage(self, filename):
        log.info("Loading tile layer from: "+filename)
        file = open(filename, "rb")
        log.debug("Image file open:"+str(file))
        #import PIL.TgaImagePlugin.TgaImageFile
        #PIL.TgaImagePlugin.TgaImageFile 
        im = Image.open(file)#source_tiles)
        
        log.debug("Loading Image file:"+str(im))

        pix = im.load()
        log.debug("Closing Image file:"+str(pix))
        file.close()
        
        width , height = im.size
        log.info("level size:"+str( im.size))
        
        packed_data = []
        
        
            
        for x in xrange(width):
            row = []
            for y in xrange(height):
                row.append(pix[x,height-1-y])
             
            packed_data.append(row)
        
        return   width , height, packed_data#pix 
