from health import Health

import random
from cocos.sprite import  Sprite
from flock import Flock

from cocos.euclid import Vector2

from pyglet import clock
import logging
log = logging.getLogger('nest')
from pyglet.event import EventDispatcher

class Nest(Sprite, Health):
    def __init__(self, world, cfg, bird_cfg):
        
        Sprite.__init__(self, cfg.filename)
        Health.__init__(self, world.quadtree, max_health = cfg.default_health, type = "nest")
        
        self.min_health =       cfg.default_health #10
        self.max_in_nest =      cfg.max_in_nest#30
        self.grow_interval =    cfg.grow_interval#3.0#5.0
        self.grow_multiplier =  cfg.grow_multiplier#1.1
        self.spawn_interval =   cfg.spawn_interval#0.3
        self.position =         cfg.position
        
        self.world = world
        self.last_brith_time = 0
        self.flock = None
        self.bird_cfg = bird_cfg
        
        log.info("Growth: "+str((self.grow_interval, self.grow_multiplier)))
        
        self.birds_leaving = False
        self.schedule_interval(self.grow, self.grow_interval)
        
        self.world.quadtree.add_item(self)
        
    def grow(self,dt):
        log.debug("Nest: growning -->:"+str(self.health))
        self.health *= self.grow_multiplier
        if self.health > self.max_in_nest:
            self.leave_nest()
        log.debug("Nest: -->growning:"+str(self.health))
        self.world.dispatch_event("on_nest_size_update")

    def die(self):
        log.info("Nest: die")
        self.unschedule(self.grow)
        self.unschedule(self.fly)
        self.parent.nest = None
        Health.die(self)  
    
    def leave_nest(self):
        if self.birds_leaving is False and self.health > self.min_health  and self.flock is None:
            # Create a new flock and start adding birds to it
            self.birds_leaving = True
            log.info("Leaving nest:"+str(self.health-self.min_health)+" spawn interval: "+str( self.spawn_interval))
            self.schedule_interval(self.fly, self.spawn_interval)
 
    def fly(self, dt):
        if self.health > self.min_health:
            if random.randint(0,5) >1:
                if self.flock is None:
                    self.flock = Flock(self.world , self.bird_cfg, 0)
                    self.flock.position = self.position
                    self.parent.AddFlock(self.flock)
                    self.flock.target = Vector2(self.x, self.y)
                    
                self.flock.AddToFlock(self.position)
                self.health -= 1
                self.world.dispatch_event("on_nest_size_update")
                
        else:
            self.birds_leaving = False
            if self.flock:
                self.flock.attached = False
            self.flock = None
            self.unschedule(self.fly)
            
 