
from complex_boid import ComplexBoid
from health import Health
from fsm import FSM
from constants import *
from splode import Splode, Smoke
from cocos.euclid import Vector2
from cocos.actions import Delay, CallFuncS, FadeOut
from cocos.sprite import Sprite
import cocos
import logging
log = logging.getLogger('plane')

from anims import GetAnimSeqFromFile, GetFramesFromFile

class Plane(ComplexBoid, Health, FSM):
    def __init__(self, sg, name, cfg):
        ComplexBoid.__init__(self, None, cfg.filename, cfg)
        Health.__init__(self, sg, type = "plane", max_health = cfg.default_health)
        FSM.__init__(self)
        
        self.name = name
        self.world_layer = None
        self.fall_accel         = cfg.fall_accel
        self.engine_ok_percent  = cfg.engine_ok_percent
        self.propellor = None
        if cfg.prop_filename != "None":
            panims = GetAnimSeqFromFile(cfg.prop_filename,cfg.prop_size, timeperframe = 0.05)
            self.propellor = Sprite(panims)
            print "prp",panims
            self.add(self.propellor)
            self.propellor.position =  cfg.prop_offset
        
        point = cfg.spawn["path"][0]
        self.target = Vector2(point[0], point[1])
        
        """
        
        self.path =  []
        for point in cfg.spawn["path"]:
            self.path.append(Vector2(point[0], point[1]))
        self.path_radius  = cfg.spawn["path_radius"]
        log.info("path:"+str(self.path))
        """
        # idle      - Normal state (flying to target)
        # stalled   - aircraft no-longer steerable - will fall to the ground
        # crashed   - lying on the ground

        self.add_state("idle",        "idle",            "idle",    self.do_idle)
        
        self.add_state("idle",      "stall",           "stalled",   self.do_fall)
        
        self.add_state("idle",      "collide_ground",  "crashed",   self.do_crashed)
        self.add_state("stalled",   "collide_ground",  "crashed",   self.do_crashed)
        self.set_state("idle")
        if DEBUG:
            self.state_label = cocos.text.Label('Plane!')
            self.add(self.state_label)
            self.schedule(self.update_state_label)
                    
    def update_state_label(self,dt):
        self.state_label.element.text = str(self.state)+" H: "+ str(self.health) +" V: "+ str(int(self.velocity.magnitude()))
                
    def PositionUpdate(self, dt):
        ComplexBoid.PositionUpdate(self, dt)
        Health.update(self, dt)
        FSM.update(self, dt)

    def VelocityUpdate(self, flocks =  None): 
        for flock in flocks:
            # Ignore flocks with less birds than us
            if len(flock.boids) < self.health:
                continue
            
            ComplexBoid.AvoidUpdate(self, flock.center)
   
    def do_idle(self):
        #log.info("do_idle "+self.type+": "+self.state+" "+str(self.pos))
        ComplexBoid.VelocityUpdate(self)
        pcnt = float(self.health)/float(self.max_health)
        if pcnt < self.engine_ok_percent:
            log.info("do_idle "+self.type+" stalling: "+str(((self.health,self.max_health),pcnt , self.engine_ok_percent)))
            self.execute_state("stall")
            smoke = Smoke()
            self.add(smoke, z = 10)
        
    def do_fall(self):
        log.info("do_fall "+self.type+": "+self.state+" "+str(self.pos))
        # we dont get scared of anything
        self.target = None
        self.acc = Vector2(self.fall_accel[0], self.fall_accel[1])
        
        
    def do_crashed(self):
        log.info("do_crashed "+self.type+": "+self.state+" "+str(self.pos))
        self.action = None
        self.target = None
        self.velocity = Vector2(0,0)
        self.velocity_dash = Vector2(0,0)
            
        log.info(self.name+ " dying "+str(self))
        
        if self in self.world_layer.aircraft:
            self.world_layer.aircraft.remove(self) 
            
        splode = Splode()  
        #splode.position = self.position
        splode.scale = 4.0
        self.add(splode, z = 10)  
        self.do(FadeOut(1.9)+CallFuncS(Health.die)) 
        if self.propellor is not None:
            self.propellor.do(FadeOut(1.9))

    def die(self):
        pass#self.execute_state("collide_ground")

