#############################################################################
#
# Copyright (c) 2008 by Casey Duncan and contributors
# All Rights Reserved.
#
# This software is subject to the provisions of the MIT License
# A copy of the license should accompany this distribution.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
#
#############################################################################
"""Explosion using textured billboard quads"""

__version__ = '$Id: splode.py 203 2009-04-05 03:52:58Z casey.duncan $'

import os
import math
import time
from pyglet import image
from pyglet.gl import *

from lepton import Particle, ParticleGroup, default_system
from lepton.renderer import BillboardRenderer
from lepton.texturizer import SpriteTexturizer, create_point_texture
from lepton.emitter import StaticEmitter
from lepton.controller import Gravity, Lifetime, Movement, Fader, ColorBlender
from lepton.renderer import PointRenderer


from lepton.renderer import PointRenderer
from lepton.texturizer import SpriteTexturizer, create_point_texture
from lepton.emitter import StaticEmitter
from lepton.controller import Gravity, Lifetime, Movement, Fader
from lepton.system import ParticleSystem
import cocos
''' 
win = pyglet.window.Window(resizable=True, visible=False)
win.clear()

def on_resize(width, height):
	"""Setup 3D projection for window"""
	glViewport(0, 0, width, height)
	glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
	gluPerspective(70, 1.0*width/height, 0.1, 1000.0)
	glMatrixMode(GL_MODELVIEW)
	glLoadIdentity()
win.on_resize = on_resize
'''

def SetupParticle():
	pyglet.clock.schedule_interval(default_system.update, (1.0/30.0))

class LeptonNode(cocos.cocosnode.CocosNode):
	def __init__(self):
		cocos.cocosnode.CocosNode.__init__(self)

class Smoke(LeptonNode):
	def __init__(self, dx=25,dy=10, gravity = (0, -2, 0), myrate = 30, spread =  (1,1,0)):
		LeptonNode.__init__(self)
		fire_tex = image.load('data/images/puff.png').get_texture()
		self.smoke = StaticEmitter(
			rate=myrate,
			template=Particle(
				position=(0,0,0), 
				velocity=(dx,dy,0), 
				color=(0.0,0.0,0.0,0.6),#0.005),
			),
			deviation=Particle(
				position=(1,1,0), 
				velocity=spread, 
				color=(0.05,0.05,0.05,0.1),#0.0),
			)
		)
		
		
		default_system.add_global_controller(
			Lifetime(4),
			Gravity(gravity), 
			Movement(), 
			Fader(fade_in_end=0, max_alpha=0.05, fade_out_start=3, fade_out_end=4),
		)
		
		self.group1 = ParticleGroup(controllers=[self.smoke],
			renderer=PointRenderer(4, SpriteTexturizer(create_point_texture(3, 1))))#fire_tex.id)))
		
		self.smoke.emit(100, self.group1)
		#self.schedule(default_system.update)
		
	def draw(self):
		self.transform()

		glEnable(GL_BLEND)
		glShadeModel(GL_SMOOTH)
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
		glDisable(GL_DEPTH_TEST)
		
		default_system.draw()
"""		
class Splode(LeptonNode):
	def __init__(self):
		LeptonNode.__init__(self)
		spark_tex = image.load( 'data/images/flare3.png').get_texture()
		fire_tex = image.load('data/images/puff.png').get_texture()
		
		self.sparks = ParticleGroup(
			controllers=[
				Lifetime(1.0),
				Movement(damping=0.93),
				Fader(fade_out_start=0.5, fade_out_end=1.0),
			],
			renderer=BillboardRenderer(SpriteTexturizer(spark_tex.id)))
		
		self.spark_emitter = StaticEmitter(
			template=Particle(
				position=(0,0,-100), 
				color=(1,1,1), 
				size=(2,2,0)),
			deviation=Particle(
				position=(1,1,1), 
				velocity=(75,75,75), 
				size=(0.2,0.2,0),
				age=1.5))
		self.spark_emitter.emit(100, self.sparks)
		
		
		self.fire = ParticleGroup(
			controllers=[
				Lifetime(2),
				Movement(damping=0.95),
				Fader(fade_in_start=0, start_alpha=0, fade_in_end=0.5, max_alpha=0.4, 
					fade_out_start=0.9, fade_out_end=1.9)
			],
			renderer=BillboardRenderer(SpriteTexturizer(fire_tex.id)))
		
		self.fire_emitter = StaticEmitter(
			template=Particle(
				position=(0,0,-100), 
				size=(20,20,0)),
			deviation=Particle(
				position=(2,2,2), 
				velocity=(20,20,20), 
				size=(5,5,0),
					up=(0,0,math.pi*2), 
					rotation=(0,0,math.pi*0.03),),
				color=[(0.5,0,0), (0.5,0.5,0.5), (0.4,0.1,0.1), (0.85,0.3,0)],
			)
		self.fire_emitter.emit(100, self.fire)
		
	def draw(self):
		#glPushMatrix()
		self.transform()

		
		#glPushAttrib(GL_COLOR_BUFFER_BIT)
		
		glEnable(GL_BLEND)
		glShadeModel(GL_SMOOTH)
		glBlendFunc(GL_SRC_ALPHA,GL_ONE)
		glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
		glDisable(GL_DEPTH_TEST)
		
		#glLoadIdentity()
		default_system.draw()
		
class Blood(LeptonNode):
	def __init__(self):
		LeptonNode.__init__(self)
		spark_tex = image.load( 'data/images/flare3.png').get_texture()
		
		
		self.sparks = ParticleGroup(
			controllers=[
				Lifetime(0.4),
				Movement(damping=0.93),
				Fader(fade_out_start=0.38, fade_out_end=0.4),
			],
			renderer=BillboardRenderer(SpriteTexturizer(spark_tex.id)))
		
		self.spark_emitter = StaticEmitter(
			template=Particle(
				position=(0,0,0), 
				color=(1,0,0), 
				size=(2,2,0)),
			deviation=Particle(
				position=(1,1,1), 
				velocity=(30,30,30), 
				size=(0.2,0.2,0),
				age=1.5))
		self.spark_emitter.emit(100, self.sparks)
		
		self.schedule(default_system.update)
		
	def draw(self):
		#glPushMatrix()
		self.transform()

		
		#glPushAttrib(GL_COLOR_BUFFER_BIT)
		
		#glEnable(GL_BLEND)
		#glShadeModel(GL_SMOOTH)
		#glBlendFunc(GL_SRC_ALPHA,GL_ONE)
		#glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
		#glDisable(GL_DEPTH_TEST)
		
		#glLoadIdentity()
		default_system.draw()
"""

''' 
@win.event
def on_draw():
	win.clear()
	glLoadIdentity()
	default_system.draw()

if __name__ == '__main__':
	win.set_visible(True)
	time.sleep(0.5)
	pyglet.clock.schedule_interval(default_system.update, (1.0/30.0))
	pyglet.app.run()
'''
		
class Splode(LeptonNode):
	def __init__(self):
		LeptonNode.__init__(self)
		spark_tex = image.load( 'data/images/flare3.png').get_texture()
		fire_tex = image.load('data/images/puff.png').get_texture()
		self.sys = ParticleSystem()
		
		self.sparks = ParticleGroup(
			controllers=[
				Lifetime(1.0),
				Movement(damping=0.93),
				Fader(fade_out_start=0.5, fade_out_end=1.0),
			],
			renderer=BillboardRenderer(SpriteTexturizer(spark_tex.id)))
		
		self.spark_emitter = StaticEmitter(
			template=Particle(
				position=(0,0,-100), 
				color=(1,1,1), 
				size=(2,2,0)),
			deviation=Particle(
				position=(1,1,1), 
				velocity=(75,75,75), 
				size=(0.2,0.2,0),
				age=1.5))
		self.spark_emitter.emit(100, self.sparks)
		
		
		self.fire = ParticleGroup(
			controllers=[
				Lifetime(2),
				Movement(damping=0.95),
				Fader(fade_in_start=0, start_alpha=0, fade_in_end=0.5, max_alpha=0.4, 
					fade_out_start=0.9, fade_out_end=1.9)
			],
			renderer=BillboardRenderer(SpriteTexturizer(fire_tex.id)))
		
		self.fire_emitter = StaticEmitter(
			template=Particle(
				position=(0,0,-100), 
				size=(20,20,0)),
			deviation=Particle(
				position=(2,2,2), 
				velocity=(20,20,20), 
				size=(5,5,0),
					up=(0,0,math.pi*2), 
					rotation=(0,0,math.pi*0.03),),
				color=[(0.5,0,0), (0.5,0.5,0.5), (0.4,0.1,0.1), (0.85,0.3,0)],
			)
		self.fire_emitter.emit(100, self.fire)
		self.sys.add_group(self.fire)
		self.schedule(self.sys.update)
		
	def draw(self):
		#glPushMatrix()
		self.transform()

		
		#glPushAttrib(GL_COLOR_BUFFER_BIT)
		
		glEnable(GL_BLEND)
		glShadeModel(GL_SMOOTH)
		glBlendFunc(GL_SRC_ALPHA,GL_ONE)
		glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
		glDisable(GL_DEPTH_TEST)
		
		#glLoadIdentity()
		self.sys.draw()#default_system.draw()
		
class Blood(LeptonNode):
	def __init__(self):
		LeptonNode.__init__(self)
		spark_tex = image.load( 'data/images/flare3.png').get_texture()
		
		self.sys = ParticleSystem()
		self.sparks = ParticleGroup(
			controllers=[
				Lifetime(0.4),
				Movement(damping=0.93),
				Fader(fade_out_start=0.28, fade_out_end=0.4),
			],
			renderer=PointRenderer(4, SpriteTexturizer(create_point_texture(3, 1))))#BillboardRenderer(SpriteTexturizer(spark_tex.id)))
		
		self.spark_emitter = StaticEmitter(
			template=Particle(
				position=(0,0,0), 
				color=(1,0,0), 
				size=(2,2,0)),
			deviation=Particle(
				position=(1,1,1), 
				velocity=(30,30,30), 
				size=(0.2,0.2,0),
				age=1.5))
		self.spark_emitter.emit(100, self.sparks)
		self.sys.add_group(self.sparks)
		self.schedule(self.sys.update)
		
	def draw(self):
		#glPushMatrix()
		self.transform()

		
		#glPushAttrib(GL_COLOR_BUFFER_BIT)
		
		#glEnable(GL_BLEND)
		#glShadeModel(GL_SMOOTH)
		#glBlendFunc(GL_SRC_ALPHA,GL_ONE)
		#glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
		#glDisable(GL_DEPTH_TEST)
		
		#glLoadIdentity()
		self.sys.draw()#default_system.draw()

