import logging
log = logging.getLogger('world')

from random import random, uniform, randrange

from cocos.layer import Layer
from cocos.sprite import Sprite
from cocos.euclid import Vector2
import cocos.rect as rect
from cocos.director import director
from pyglet import clock

from pixel_collide import collide, SpriteCollision#, Rect

from extern import scenegraph

from flock import Flock
from city import City
from plane import Plane
from nest import Nest
from level_background import Level
from item_cfg import BuildItemConfigTable
from extern.primitives import Line
from pyglet.event import EventDispatcher
from splode import SetupParticle          
import time

class World(Layer, EventDispatcher):
    """ 
    Look after clouds / planes / world interaction
    """
    is_event_handler = True # for actions to work
    def __init__(self, levelname):
        
        Layer.__init__(self)
        EventDispatcher.__init__(self)
        SetupParticle()
        t = time.time()
        self.cfg =  BuildItemConfigTable("levels/"+levelname)
        ct = time.time() - t
        log.info( 'BuildItemConfigTable took: %f' % ct)
       
        self.MIN_FLOCK_MERGE_DISTANCE = self.cfg.world["flock_merge_dist"] # 50

        t = time.time()
        self.level = Level(tile_size = self.cfg.world["tile_size"],
                      data_file = self.cfg.world["level_data"],
                      tile_file = self.cfg.world["level_tiles"])
        lt = time.time() - t
        log.info( 'Level took: %f' % lt)
        log.info("Level size:"+str((self.level.maplayer.px_width, self.level.maplayer.px_height)))
        
        self.level.maplayer.add( self )
        self.level.maplayer.opacity = 240

        self.state = "running"

        self.quadtree = scenegraph.Node(bounds=rect.Rect(0, 0, self.level.maplayer.px_width, self.level.maplayer.px_height))
      

        self.cities = []
        self.aircraft = []
        self.aircraft_departures = {}
        self.flocks = []
        self.birds = []
        self.humans = []
        self.clouds = []
        
        self.no_flock = False
        # clouds / non interactive movables
        self.misc = [] 
        self.win = False
        self.aircraft_spawn = Vector2(self.level.maplayer.px_width+900, 320)
        
        t = time.time()        
        # Add some offsets to make them all beat at different rate
        self.flock = Flock(self, self.cfg.birds, self.cfg.world["start_flock_size"])#10) 
        lt = time.time() - t
        log.info( 'Creating flock took: %f' % lt)
        
        self.nest = Nest(self, self.cfg.nest, self.cfg.birds)    

        
        
        for name in self.cfg.cities.keys():
            city = City(self, name, self.cfg.cities[name], self.cfg.humans)
            self.AddCity(city)
 
        #self.TellCities("crash")
        #self.TellCities("crash")
        
        #self.cities[0].do_spawn()
        """ 
        self.cities[0].hurt()
        
        self.cities[1].hurt()
        self.cities[1].hurt()
        
        self.cities[2].hurt()
        self.cities[2].hurt()
        self.cities[2].hurt()
        
        self.cities[3].hurt()
        self.cities[3].hurt()
        self.cities[3].hurt()
        self.cities[3].hurt()
        """
        ''' 
        self.plane = Plane(self.quadtree, "LargePlane", self.cfg.planes["LargePlane"])
        self.plane.position = 400, 400#
        self.add(self.plane)
        '''
        self.add(self.flock)
        self.add(self.nest)
        
        self.CreateDepartureSchedule()
        
        #self.schedule_interval(self.SchedualAircraft, self.cfg.world["schedual_aircraft"])
         
        self.AddCloudLayers()
        self.AddTrees(self.cfg.world["trees"], self.cfg.world["tree_file"])
       
        self.mouse = Vector2(300,300)
        self.shoot_lines = []
        
        self.dispatch_event("on_flock_size_update")
        self.dispatch_event("on_nest_size_update")
        
        self.schedule(self.update)

    def AddTrees(self, trees, file):
        for pos in trees:
            tree = Sprite(file)
            tree.position = pos
            self.level.maplayer.add(tree, z=11)
            
    def AddCloudLayers(self):
        width, height = self.level.maplayer.px_width, self.level.maplayer.px_height
        self.clouds = []
        for n in xrange(10):
            id = randrange(1,14)
            x = uniform(0, width)
            y = uniform(300, height)
            depth = uniform(0, 2)
            cloud = Sprite("data/mini_images/cloud"+str(id)+".png")
            cloud.position=(x,y)

            self.clouds.append(cloud)
            self.level.maplayer.add(cloud, z=depth*11)
 
    def update_clouds(self, dt):
        for cloud in self.clouds:
            cloud.x = (cloud.x+256 + 10*dt)%self.level.maplayer.px_width-256
       
    def on_mouse_motion(self, x, y, dx, dy):
        self.mouse = Vector2(x,y)
    
    def draw(self):
        Layer.draw(self)
        for line in self.shoot_lines:
            line.render()
                
    def update(self, dt):
        self.shoot_lines = []
        log.debug("update "+str(dt))
        x,y = self.level.scroller.pixel_from_screen(self.mouse.x,self.mouse.y)
        
        # Update player pos
        self.flock.target = Vector2(x,y)
        self.flock.UpdateFlock(dt)
        self.flock.UpdateFlockCenter()
        
        # Update view of level
        self.level.SetFocus(self.flock.center.x,self.flock.center.y)
        
        # Update un attached flocks
        for flock in self.flocks:
            flock.UpdateFlock(dt)
            flock.UpdateFlockCenter()
            if (self.flock.center - flock.center).magnitude() < self.MIN_FLOCK_MERGE_DISTANCE:
                self.flock.Merge(flock)
                self.remove(flock)
                self.flocks.remove(flock)
                if self.nest is not None and self.nest.flock is flock:
                    self.nest.flock = None
                self.dispatch_event("on_flock_size_update")
         
        # Update Aircraft       
        self.UpdateAircraft(dt)#, self.flock)
        self.CheckWorldCollisions()
        
        if len(self.flock.boids) == 0 and self.no_flock is False:
            self.no_flock = True
            self.dispatch_event("on_game_over")
            
        # Update cities
        for city in self.cities:
            city.update(dt)
        
        self.SchedualAircraft(dt)
        
        #self.plane.position = 400, 400#
        
        self.update_clouds(dt)
        
        if len(self.cities) == 0 and self.win is False :
            print "WIN!"
            self.win = True
            self.dispatch_event("on_game_win")

    #####################################################################################
    ################################    Cities    #######################################
    #####################################################################################

    def AddCity(self, city):
        log.info(self.state+": Adding city "+city.name)

        city.set_state(self.cfg.world["default_city_state"])      
        city.nest = self.nest
        self.add(city)

        self.cities.append(city)
        #self.level.maplayer.add(city)
        
    def RemoveCity(self, city):
        log.info(self.state+": Removing city "+city.name)
        self.cities.remove(city)
        #self.level.maplayer.remove(city)

    def TellCities(self, event):
        for city in self.cities:
            city.execute_state(event)
        
    #####################################################################################
    ################################    Flocks    #######################################
    #####################################################################################

    def AddFlock(self, flock):
        self.flocks.append(flock)
        self.add(flock)

    #####################################################################################
    ################################    Aircraft    #####################################
    #####################################################################################

    def SetNextDeparture(self, aircraft):
        time = clock.time.time()
        plane_spawn = self.cfg.planes[aircraft].spawn
        fromnow = plane_spawn["frequency_base"] + random() * plane_spawn["frequency_range"]
        self.aircraft_departures[aircraft] = time + fromnow
        log.info(self.state+": Next departing "+aircraft+" "+str(fromnow)+" from now: "+str(self.aircraft_departures[aircraft]))

         
    def CreateDepartureSchedule(self):
        for type in self.cfg.planes.keys():
            self.SetNextDeparture(type)
            
    def SchedualAircraft(self, dt):
        time = clock.time.time()
        for type in self.cfg.planes.keys():
            if time > self.aircraft_departures[type]:
                log.info(self.state+": Aircraft departing "+str(type)+" "+str((self.aircraft_departures[type], time)))
                self.SetNextDeparture(type)
                plane = Plane(self.quadtree, str(type), self.cfg.planes[type])
                plane.pos = self.aircraft_spawn.copy()#Vector2(400,400)#
                plane.position =self.aircraft_spawn.x,self.aircraft_spawn.y# 400, 400#
                #plane.target = Vector2(-500,500)#Vector2(400,30)#
                self.AddAircraft(plane)
        
    def AddAircraft(self, aircraft):
        log.info(self.state+": Adding aircraft "+aircraft.name +" " +str(aircraft.pos))

        aircraft.world_layer = self
        self.aircraft.append(aircraft)
        self.level.maplayer.add(aircraft)
        
    def RemoveAircraft(self, aircraft):
        log.info(self.state+": Removing aircraft "+aircraft.name +" "+str(aircraft.pos))
        self.aircraft.remove(aircraft)
        self.level.maplayer.remove(aircraft)
        
    def UpdateAircraft(self, dt):

        for plane in self.aircraft:
            
            log.debug("plane:"+str(plane.pos))
            plane.VelocityUpdate(self.flocks)
            plane.PositionUpdate(dt)
            
            if plane.pos.x < -100:
                self.RemoveAircraft(plane)


    #####################################################################################
    ##########################    Collision Response    #################################
    #####################################################################################

    def AircraftCrash(self, aircraft):
        self.TellCities("crash")
        aircraft.execute_state("collide_ground")
        self.dispatch_event("on_kill_aircraft")
        self.AircraftExplode(aircraft)
        
    def AircraftExplode(self, aircraft):
        # explode :)
        bp = rect.Rect(int(aircraft.x-aircraft.width/2),
                       int(aircraft.y-aircraft.height/2),
                       aircraft.width*3,aircraft.height*3)
        
        possibles = self.quadtree.hit(bp)
        for item in possibles:
            if item is aircraft:
                continue
            if item.dead is True:
                continue
            if item.type == "human" or item.type == "bird":
                pos = Vector2(item.position[0], item.position[1])
                if (pos-aircraft.pos).magnitude() < 150:
                    log.info("An aircraft has crashed effecting:"+str(item.type))
                    if item.health >0:
                        item.die()
        

    def AircraftVSCity(self, aircraft, city):
        log.info("An aircraft has crashed into a city!"+str(aircraft))
        self.TellCities("crash_city")
        city.hurt()
        aircraft.execute_state("collide_ground")
        self.dispatch_event("on_kill_aircraft")
        self.AircraftExplode(aircraft)
        
    def BirdVSCity(self, bird, city):
        self.shoot_lines.append( Line( a=bird.position, b=city.position, z=20, color=(0.2,0,0,1))    )

        self.TellCities("kill_bird")
        bird.die()
        self.dispatch_event("on_kill_bird")
        
    def AircraftVSBird(self, aircraft, bird):
        log.info("Attacking an aircraft!")
        
        if aircraft.hurt():
            log.info("Killed an aircraft!")
            self.TellCities("crash")
            self.dispatch_event("on_kill_aircraft")
            aircraft.execute_state("stall")
            #self.AircraftExplode(aircraft)
        else:
            # chunky!!
            bird.die()                               

    def HumanVSBird(self, human, bird):
        log.info("Attacking a human!")
        if human.hurt():
            log.info("Killed a human!")
            bird.kills += 1
            self.TellCities("death")
            self.dispatch_event("on_kill_human")

    #####################################################################################
    ###############################    Collision    #####################################
    #####################################################################################

        
    def CheckWorldCollisions(self):
        time = clock.time.time()
        
        # Static stuff first - cities
        for city in self.cities:
            if city.dead is True:
                continue
            
            x = city.x - city.attack_distance/2
            
            bp = rect.Rect(int(x),int(city.y),city.width+city.attack_distance,city.height+city.attack_distance)
            possibles = self.quadtree.hit(bp)
            for item in possibles:
                if item.dead is True:
                    continue
                
                # Ignore humans
                if item.type == "human":
                    pass
                
                elif item.type == "bird": # If its a bird
                    # did it collide?
                    # City vs Birds - birds no hope
                    if collide(item.collision, city.collision):
                        log.info("A bird got too close to a city!")
                        self.BirdVSCity(item, city)
                    else:
                        # is it within range?
                        # If the bird is within range of the city - take a shot if we can
                        if time > city.last_attack_time + city.attack_frequency:
                            dist = Vector2(item.pos.x - city.x, item.pos.y - city.y).magnitude()
                            if dist < city.attack_distance:
                                city.last_attack_time = time
                                log.info("A city is attacking a bird!")
                                self.BirdVSCity(item, city)
                    
                elif item.type == "plane": # If its a plane
                    # did it collide? 
                    if collide(item.collision, city.collision):
                        self.AircraftVSCity(item, city)
                        
                        
            # Humans attack birds
            if len(city.humans) >0:
    
                for human in city.humans:
                    if human.dead is True:
                        continue
                    x = human.x - human.attack_distance
                    
                    bp = rect.Rect(int(x),int(human.y),human.width+human.attack_distance*2,human.height+human.attack_distance)
                    possibles = self.quadtree.hit(bp)
                    for item in possibles:
                        if item.dead is True:
                            continue
                        if item.type == "bird":
                            human.set_closest_bird(item)
            
                    
               
        # Bird VS human set the bird in the humans sights
        for bref in self.birds:
            bird = bref()
            if bird.dead is True:
                self.birds.remove(bref)
                continue
            
            bp = rect.Rect(int(bird.x),int(bird.y),bird.width,bird.height)
            possibles = self.quadtree.hit(bp)
            for item in possibles:
                
                # ignore dead things
                if item.dead is True:
                    continue
                
                # Ignore birds
                if item.type == "bird":
                    pass
                elif item.type == "city":
                    pass # should have already been delt with above
                
                elif item.type == "plane":
                    if collide(item.collision, bird.collision):
                        self.AircraftVSBird(item, bird)
                        self.birds.remove(bref)
                        break
                        
                
                elif item.type == "human":
                    # Birds attack humans
                    if time - bird.last_attack_time > 1:
                        bird.last_attack_time = time
                        if collide(item.collision, bird.collision):
                            self.HumanVSBird(item, bird)
                            
                # we cant continue if we have died            
                if bird.dead is True:
                    self.birds.remove(bref)
                    break
        
        # Aircraft VS Ground        
        for plane in self.aircraft:
            # We dead - no point....
            if plane.dead is True:
                self.aircraft.remove(plane)
                continue
            
            # Aircraft Vs ground            
            plane_rect = plane.collision.get_rect() # get tiles near aircraft
            cells = self.level.maplayer.get_in_region(plane_rect.x1, plane_rect.y1, plane_rect.x2, plane_rect.y2)
            #log.info("cell count:"+str(len(cells))+" plane: "+str(rect)+str(plane.position)) 
            for cell in cells:
                if cell.tile == self.level.notile.tile:
                    #log.info("Skip cell "+str(cell))
                    continue

                if collide(plane.collision, cell.tile.collision, cell.position):#cell.tile.collision):
                    log.info("An aircraft has crashed! "+str(plane))
                    self.AircraftCrash(plane)
                    break
                    #self.plane = None


World.register_event_type('on_kill_human')                                   
World.register_event_type('on_kill_aircraft') 

World.register_event_type('on_kill_bird') 
World.register_event_type('on_kill_nest') 
World.register_event_type('on_attack_nest') 
World.register_event_type('on_flock_size_update')  
World.register_event_type('on_nest_size_update')  
                              
World.register_event_type('on_game_over')   
World.register_event_type('on_game_win')   

